/****************************************************************************

 MESSAGE PASSING INTERFACE TEST CASE SUITE

 Copyright IBM Corp. 1995

 IBM Corp. hereby grants a non-exclusive license to use, copy, modify, and
 distribute this software for any purpose and without fee provided that the
 above copyright notice and the following paragraphs appear in all copies.

 IBM Corp. makes no representation that the test cases comprising this
 suite are correct or are an accurate representation of any standard.

 In no event shall IBM be liable to any party for direct, indirect, special
 incidental, or consequential damage arising out of the use of this software
 even if IBM Corp. has been advised of the possibility of such damage.

 IBM CORP. SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED
 TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS AND IBM
 CORP. HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 ENHANCEMENTS, OR MODIFICATIONS.

****************************************************************************

 These test cases reflect an interpretation of the MPI Standard.  They are
 are, in most cases, unit tests of specific MPI behaviors.  If a user of any
 test case from this set believes that the MPI Standard requires behavior
 different than that implied by the test case we would appreciate feedback.

 Comments may be sent to:
    Richard Treumann
    treumann@kgn.ibm.com

****************************************************************************
*/

//#include "mpi.h"
import mpi.*;

class isend {

  static int tasks,bytes,i; //me,data[1000],buf[10000];
  static int me[] = new int[1];
  static int data[] = new int[1000];
  //static int buf[] = new int[10000];
  static byte buf[] = new byte[10000];
  //MPI_Request req[1000];
  static Request req[];
  //MPI_Status stats[1000];
  static Status stats[];
  
  static void wstart()
  {
    //MPI_Waitall(2*tasks,req,stats);
    stats = Request.Waitall(req);
    
    for(i=0;i<tasks;i++)
      if(data[i] != i)
	System.out.println
	  ("ERROR in Startall: data is"+data[i]+", should be"+i);

    /* ONLY THE RECEIVERS HAVE STATUS VALUES ! */
    for(i=1;i<2*tasks;i+=2) {
      //MPI_Get_count(&stats[i],MPI_BYTE,&bytes);
      bytes = stats[i].Get_count(MPI.BYTE); 
      if(bytes != 4)
	System.out.println
	  ("ERROR in Waitall: bytes ="+bytes+", should be 4");
    }
  }
  

  public static void main(String[] args) {
    //MPI_Init(0,0);
    //MPI_Comm_rank(MPI_COMM_WORLD,&me);
    //MPI_Comm_size(MPI_COMM_WORLD,&tasks);
    MPI.Init(args);
    me[0]=MPI.COMM_WORLD.Rank();
    tasks=MPI.COMM_WORLD.Size();

    req = new Request[2*tasks];
    stats = new Status[2*tasks];  


    //MPI_Buffer_attach(buf,sizeof(buf));
    MPI.Buffer_attach(buf);


    for(i=0;i<tasks;i++)  data[i] = -1;
    for(i=0;i<tasks;i++)  {
      //MPI_Isend(&me,1,MPI_INT,i,1,MPI_COMM_WORLD,&req[2*i]);
      req[2*i]=MPI.COMM_WORLD.Isend(me,0,1,MPI.INT,i,1);
      //MPI_Irecv(&data[i],1,MPI_INT,i,1,MPI_COMM_WORLD,&req[2*i+1]);
      req[2*i+1]=MPI.COMM_WORLD.Irecv(data,i,1,MPI.INT,i,1);
      //MPI.COMM_WORLD.Irecv(data,i,1,MPI.INT,i,1,req[2*i+1]);
    }
    wstart();
    

    for(i=0;i<tasks;i++)  data[i] = -1;
    for(i=0;i<tasks;i++)  {
      //MPI_Issend(&me,1,MPI_INT,i,1,MPI_COMM_WORLD,&req[2*i]);
      req[2*i]=MPI.COMM_WORLD.Issend(me,0,1,MPI.INT,i,1);
      //MPI_Irecv(&data[i],1,MPI_INT,i,1,MPI_COMM_WORLD,&req[2*i+1]);
      req[2*i+1]=MPI.COMM_WORLD.Irecv(data,i,1,MPI.INT,i,1);
    }
    wstart();
    

    for(i=0;i<tasks;i++)  data[i] = -1;
    for(i=0;i<tasks;i++)  
      //MPI_Irecv(&data[i],1,MPI_INT,i,1,MPI_COMM_WORLD,&req[2*i+1]);
      req[2*i+1]=MPI.COMM_WORLD.Irecv(data,i,1,MPI.INT,i,1);
    //MPI_Barrier(MPI_COMM_WORLD);
    MPI.COMM_WORLD.Barrier();
    for(i=0;i<tasks;i++)
      //MPI_Irsend(&me,1,MPI_INT,i,1,MPI_COMM_WORLD,&req[2*i]);
      req[2*i]=MPI.COMM_WORLD.Irsend(me,0,1,MPI.INT,i,1);
    wstart();
    

    for(i=0;i<tasks;i++)  data[i] = -1;
    for(i=0;i<tasks;i++)  {
      //MPI_Ibsend(&me,1,MPI_INT,i,1,MPI_COMM_WORLD,&req[2*i]);
      req[2*i]=MPI.COMM_WORLD.Ibsend(me,0,1,MPI.INT,i,1);
      //MPI_Irecv(&data[i],1,MPI_INT,i,1,MPI_COMM_WORLD,&req[2*i+1]);
      req[2*i+1]=MPI.COMM_WORLD.Irecv(data,i,1,MPI.INT,i,1);
    }
    wstart(); 
    
    
    MPI.COMM_WORLD.Barrier();
    if(me[0] == 0)  System.out.println("TEST COMPLETE\n");
    MPI.Finalize();
  }
}



